@@ -46,10 +46,10 @@ Colours_setup_t *Colours_setup;
 COLOURS_EXTERNAL_t *Colours_external;
 
 static Colours_setup_t const presets[] = {
-	/* Saturation, Contrast, Brightness, Gamm adjustment, Black level, White level */
-	{ 0.0, 0.0, 0.0, 0.3, 16, 235 }, /* Standard preset */
-	{ 0.0, 0.2, -0.16, 0.5, 16, 235 }, /* Deep blacks preset */
-	{ 0.26, 0.72, -0.16, 0.16, 16, 235 } /* Vibrant colours & levels preset */
+	/* Hue, Saturation, Contrast, Brightness, Gamma adjustment, GTIA delay, Black level, White level */
+	{ 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 16, 235 }, /* Standard preset */
+	{ 0.0, 0.0, 0.2, -0.16, 0.5, 0.0, 16, 235 }, /* Deep blacks preset */
+	{ 0.0, 0.26, 0.72, -0.16, 0.16, 0.0, 16, 235 } /* Vibrant colours & levels preset */
 };
 static char const * const preset_cfg_strings[COLOURS_PRESET_SIZE] = {
 	"STANDARD",
@@ -78,7 +78,7 @@ void Colours_SetRGB(int i, int r, int g, int b, int *colortable_ptr)
 
 static void UpdateModeDependentPointers(int tv_mode)
 {
-	/* Set pointers to the currnt setup and external palette */
+	/* Set pointers to the current setup and external palette. */
 	if (tv_mode == Atari800_TV_NTSC) {
 		Colours_setup = &COLOURS_NTSC_setup;
 		Colours_external = &COLOURS_NTSC_external;
@@ -88,7 +88,7 @@ static void UpdateModeDependentPointers(int tv_mode)
 		Colours_external = &COLOURS_PAL_external;
 	}
 	else {
-		Atari800_Exit(FALSE);
+		Atari800_ErrExit();
 		Log_print("Interal error: Invalid Atari800_tv_mode\n");
 		exit(1);
 	}
@@ -132,19 +132,19 @@ void Colours_Update(void)
 
 void Colours_RestoreDefaults(void)
 {
-	*Colours_setup = presets[COLOURS_PRESET_STANDARD];
-	if (Atari800_tv_mode == Atari800_TV_NTSC)
-		COLOURS_NTSC_RestoreDefaults();
+	Colours_SetPreset(COLOURS_PRESET_STANDARD);
 }
 
 /* Sets the video calibration profile to the user preference */
 void Colours_SetPreset(Colours_preset_t preset)
 {
-	if (preset < COLOURS_PRESET_CUSTOM)
+	if (preset < COLOURS_PRESET_CUSTOM) {
 		*Colours_setup = presets[preset];
-
-	if (Atari800_tv_mode == Atari800_TV_NTSC) 
-		COLOURS_NTSC_SetPreset(COLOURS_PRESET_STANDARD);
+		if (Atari800_tv_mode == Atari800_TV_NTSC) 
+			COLOURS_NTSC_RestoreDefaults();
+		else
+			COLOURS_PAL_RestoreDefaults();
+	}
 }
 
 /* Compares the current settings to the available calibration profiles
@@ -153,12 +153,15 @@ Colours_preset_t Colours_GetPreset(void)
 {
 	int i;
 
-	if (Atari800_tv_mode == Atari800_TV_NTSC &&
-	    COLOURS_NTSC_GetPreset() != COLOURS_PRESET_STANDARD)
+	if ((Atari800_tv_mode == Atari800_TV_NTSC &&
+	     COLOURS_NTSC_GetPreset() != COLOURS_PRESET_STANDARD) ||
+	    (Atari800_tv_mode == Atari800_TV_PAL &&
+	     COLOURS_PAL_GetPreset() != COLOURS_PRESET_STANDARD))
 		return COLOURS_PRESET_CUSTOM;
 
 	for (i = 0; i < COLOURS_PRESET_SIZE; i ++) {
-		if (Util_almostequal(Colours_setup->saturation, presets[i].saturation, 0.001) &&
+		if (Util_almostequal(Colours_setup->hue, presets[i].hue, 0.001) &&
+		    Util_almostequal(Colours_setup->saturation, presets[i].saturation, 0.001) &&
 		    Util_almostequal(Colours_setup->contrast, presets[i].contrast, 0.001) &&
 		    Util_almostequal(Colours_setup->brightness, presets[i].brightness, 0.001) &&
 		    Util_almostequal(Colours_setup->gamma, presets[i].gamma, 0.001) &&
@@ -199,7 +202,8 @@ void Colours_PreInitialise(void)
 {
 	/* Copy the default setup for both NTSC and PAL. */
 	COLOURS_NTSC_setup = COLOURS_PAL_setup = presets[COLOURS_PRESET_STANDARD];
-	COLOURS_NTSC_PreInitialise();
+	COLOURS_NTSC_RestoreDefaults();
+	COLOURS_PAL_RestoreDefaults();
 }
 
 int Colours_ReadConfig(char *option, char *ptr)
@@ -247,6 +251,11 @@ int Colours_Initialise(int *argc, char *argv[])
 				COLOURS_NTSC_setup.gamma = COLOURS_PAL_setup.gamma = atof(argv[++i]);
 			else a_m = TRUE;
 		}
+		else if (strcmp(argv[i], "-tint") == 0) {
+			if (i_a)
+				COLOURS_NTSC_setup.hue = COLOURS_PAL_setup.hue = atof(argv[++i]);
+			else a_m = TRUE;
+		}
 		else if (strcmp(argv[i], "-colors-preset") == 0) {
 			if (i_a) {
 				int idx = CFG_MatchTextParameter(argv[++i], preset_cfg_strings, COLOURS_PRESET_SIZE);
@@ -255,7 +264,8 @@ int Colours_Initialise(int *argc, char *argv[])
 					return FALSE;
 				}
 				COLOURS_NTSC_setup = COLOURS_PAL_setup = presets[idx];
-				COLOURS_NTSC_SetPreset(COLOURS_PRESET_STANDARD);
+				COLOURS_NTSC_RestoreDefaults();
+				COLOURS_PAL_RestoreDefaults();
 			} else a_m = TRUE;
 		}
 
@@ -267,6 +277,7 @@ int Colours_Initialise(int *argc, char *argv[])
 				Log_print("\t-contrast <num>        Set contrast");
 				Log_print("\t-brightness <num>      Set brightness");
 				Log_print("\t-gamma <num>           Set color gamma factor");
+				Log_print("\t-tint <num>            Set tint");
 			}
 			argv[j++] = argv[i];
 		}
