@@ -13,6 +13,10 @@ extern COLOURS_EXTERNAL_t COLOURS_PAL_external;
    or loading/unloading an external palette. */
 void COLOURS_PAL_Update(int colourtable[256]);
 
+/* Restores default values for PAL-specific colour controls.
+   Colours_PAL_Update should be called afterwards to apply changes. */
+void COLOURS_PAL_RestoreDefaults(void);
+
 /* Read/write to configuration file. */
 int COLOURS_PAL_ReadConfig(char *option, char *ptr);
 void COLOURS_PAL_WriteConfig(FILE *fp);
@@ -20,4 +24,7 @@ void COLOURS_PAL_WriteConfig(FILE *fp);
 /* PAL Colours initialisation and processing of command-line arguments. */
 int COLOURS_PAL_Initialise(int *argc, char *argv[]);
 
+/* Function for getting the PAL-specific color preset. */
+Colours_preset_t COLOURS_PAL_GetPreset(void);
+
 #endif /* COLOURS_PAL_H_ */
