@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Atari800; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /*
@@ -45,7 +45,7 @@
 
 /* customize this logging routine as needed */
 #include "log.h"
-#define  log_printf  Aprint
+#define  log_printf  Log_print
 
 #include "dos_sb.h"
 #include "dos_ints.h"
