@@ -127,13 +127,13 @@ int collision_curpos;
  */
 int hitclr_pos;
 #else
-#define P1PL_T P1PL
-#define P2PL_T P2PL
-#define P3PL_T P3PL
-#define M0PL_T M0PL
-#define M1PL_T M1PL
-#define M2PL_T M2PL
-#define M3PL_T M3PL
+#define P1PL_T GTIA_P1PL
+#define P2PL_T GTIA_P2PL
+#define P3PL_T GTIA_P3PL
+#define M0PL_T GTIA_M0PL
+#define M1PL_T GTIA_M1PL
+#define M2PL_T GTIA_M2PL
+#define M3PL_T GTIA_M3PL
 #endif /* NEW_CYCLE_EXACT */
 
 static UBYTE *hposp_ptr[4];
@@ -431,7 +431,7 @@ void GTIA_Frame(void)
 	}
 }
 
-UBYTE GTIA_GetByte(UWORD addr)
+UBYTE GTIA_GetByte(UWORD addr, int no_side_effects)
 {
 	switch (addr & 0x1f) {
 	case GTIA_OFFSET_M0PF:
@@ -520,7 +520,7 @@ UBYTE GTIA_GetByte(UWORD addr)
 	case GTIA_OFFSET_CONSOL:
 		{
 			UBYTE byte = GTIA_consol_table[GTIA_consol_index] & consol_mask;
-			if (GTIA_consol_index > 0) {
+			if (!no_side_effects && GTIA_consol_index > 0) {
 				GTIA_consol_index--;
 				if (GTIA_consol_index == 0 && CASSETTE_hold_start) {
 					/* press Space after Start to start cassette boot */
@@ -1203,9 +1203,10 @@ void GTIA_StateSave(void)
 	StateSav_SaveUBYTE(&consol_mask, 1);
 	StateSav_SaveINT(&GTIA_speaker, 1);
 	StateSav_SaveINT(&next_console_value, 1);
+	StateSav_SaveUBYTE(GTIA_TRIG_latch, 4);
 }
 
-void GTIA_StateRead(void)
+void GTIA_StateRead(UBYTE version)
 {
 	int next_console_value;	/* ignored */
 
@@ -1255,6 +1256,8 @@ void GTIA_StateRead(void)
 	StateSav_ReadUBYTE(&consol_mask, 1);
 	StateSav_ReadINT(&GTIA_speaker, 1);
 	StateSav_ReadINT(&next_console_value, 1);
+	if (version >= 7)
+		StateSav_ReadUBYTE(GTIA_TRIG_latch, 4);
 
 	GTIA_PutByte(GTIA_OFFSET_HPOSP0, GTIA_HPOSP0);
 	GTIA_PutByte(GTIA_OFFSET_HPOSP1, GTIA_HPOSP1);
