@@ -38,7 +38,7 @@
 extern UBYTE MEMORY_mem[65536 + 2];
 
 /* RAM size in kilobytes.
-   Valid values for Atari800_MACHINE_OSA and Atari800_MACHINE_OSB are: 16, 48, 52.
+   Valid values for Atari800_MACHINE_800 are: 16, 48, 52.
    Valid values for Atari800_MACHINE_XLXE are: 16, 64, 128, 192, RAM_320_RAMBO,
    RAM_320_COMPY_SHOP, 576, 1088.
    The only valid value for Atari800_MACHINE_5200 is 16. */
@@ -53,7 +53,11 @@ extern int MEMORY_ram_size;
 #ifndef PAGED_ATTRIB
 
 extern UBYTE MEMORY_attrib[65536];
-#define MEMORY_GetByte(addr)		(MEMORY_attrib[addr] == MEMORY_HARDWARE ? MEMORY_HwGetByte(addr) : MEMORY_mem[addr])
+/* Reads a byte from ADDR. Can potentially have side effects, when reading
+   from hardware area. */
+#define MEMORY_GetByte(addr)		(MEMORY_attrib[addr] == MEMORY_HARDWARE ? MEMORY_HwGetByte(addr, FALSE) : MEMORY_mem[addr])
+/* Reads a byte from ADDR, but without any side effects. */
+#define MEMORY_SafeGetByte(addr)		(MEMORY_attrib[addr] == MEMORY_HARDWARE ? MEMORY_HwGetByte(addr, TRUE) : MEMORY_mem[addr])
 #define MEMORY_PutByte(addr, byte)	 do { if (MEMORY_attrib[addr] == MEMORY_RAM) MEMORY_mem[addr] = byte; else if (MEMORY_attrib[addr] == MEMORY_HARDWARE) MEMORY_HwPutByte(addr, byte); } while (0)
 #define MEMORY_SetRAM(addr1, addr2) memset(MEMORY_attrib + (addr1), MEMORY_RAM, (addr2) - (addr1) + 1)
 #define MEMORY_SetROM(addr1, addr2) memset(MEMORY_attrib + (addr1), MEMORY_ROM, (addr2) - (addr1) + 1)
@@ -61,12 +65,17 @@ extern UBYTE MEMORY_attrib[65536];
 
 #else /* PAGED_ATTRIB */
 
-typedef UBYTE (*MEMORY_rdfunc)(UWORD addr);
+typedef UBYTE (*MEMORY_rdfunc)(UWORD addr, int no_side_effects);
 typedef void (*MEMORY_wrfunc)(UWORD addr, UBYTE value);
 extern MEMORY_rdfunc MEMORY_readmap[256];
+extern MEMORY_rdfunc MEMORY_safe_readmap[256];
 extern MEMORY_wrfunc MEMORY_writemap[256];
 void MEMORY_ROM_PutByte(UWORD addr, UBYTE byte);
-#define MEMORY_GetByte(addr)		(MEMORY_readmap[(addr) >> 8] ? (*MEMORY_readmap[(addr) >> 8])(addr) : MEMORY_mem[addr])
+/* Reads a byte from ADDR. Can potentially have side effects, when reading
+   from hardware area. */
+#define MEMORY_GetByte(addr)		(MEMORY_readmap[(addr) >> 8] ? (*MEMORY_readmap[(addr) >> 8])(addr, FALSE) : MEMORY_mem[addr])
+/* Reads a byte from ADDR, but without any side effects. */
+#define MEMORY_SafeGetByte(addr)		(MEMORY_readmap[(addr) >> 8] ? (*MEMORY_readmap[(addr) >> 8])(addr, TRUE) : MEMORY_mem[addr])
 #define MEMORY_PutByte(addr,byte)	(MEMORY_writemap[(addr) >> 8] ? ((*MEMORY_writemap[(addr) >> 8])(addr, byte), 0) : (MEMORY_mem[addr] = byte))
 #define MEMORY_SetRAM(addr1, addr2) do { \
 		int i; \
@@ -87,6 +96,7 @@ void MEMORY_ROM_PutByte(UWORD addr, UBYTE byte);
 
 extern UBYTE MEMORY_basic[8192];
 extern UBYTE MEMORY_os[16384];
+extern UBYTE MEMORY_xegame[8192];
 
 extern int MEMORY_xe_bank;
 extern int MEMORY_selftest_enabled;
@@ -94,6 +104,8 @@ extern int MEMORY_selftest_enabled;
 extern int MEMORY_have_basic;
 extern int MEMORY_cartA0BF_enabled;
 
+/* Verifies if SIZE is a correct value for RAM size. */
+int MEMORY_SizeValid(int size);
 void MEMORY_InitialiseMachine(void);
 void MEMORY_StateSave(UBYTE SaveVerbose);
 void MEMORY_StateRead(UBYTE SaveVerbose, UBYTE StateVersion);
@@ -108,15 +120,16 @@ void MEMORY_CartA0bfEnable(void);
 void MEMORY_GetCharset(UBYTE *cs);
 
 /* Mosaic and Axlon 400/800 RAM extensions */
-extern int MEMORY_mosaic_maxbank;
-extern int MEMORY_mosaic_enabled;
-extern int MEMORY_axlon_enabled;
+extern int MEMORY_mosaic_num_banks;
 extern int MEMORY_axlon_0f_mirror;
-extern int MEMORY_axlon_bankmask;
+extern int MEMORY_axlon_num_banks;
+
+/* Controls presence of MapRAM memory modification for XL/XE mode. */
+extern int MEMORY_enable_mapram;
 
 #ifndef PAGED_MEM
 /* Reads a byte from the specified special address (not RAM or ROM). */
-UBYTE MEMORY_HwGetByte(UWORD addr);
+UBYTE MEMORY_HwGetByte(UWORD addr, int safe);
 
 /* Stores a byte at the specified special address (not RAM or ROM). */
 void MEMORY_HwPutByte(UWORD addr, UBYTE byte);
