@@ -22,6 +22,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+#define _POSIX_C_SOURCE 200112L /* for snprintf */
+
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -167,8 +169,7 @@ char SIO_status[256];
 #define SIO_WriteFrame      (0x04)
 #define SIO_FinalStatus     (0x05)
 #define SIO_FormatFrame     (0x06)
-#define SIO_CasRead         (0x60)
-#define SIO_CasWrite        (0x61)
+#define SIO_CasReadWrite    (0x60)
 static UBYTE CommandFrame[6];
 static int CommandIndex = 0;
 static UBYTE DataBuffer[256 + 3];
@@ -710,7 +711,7 @@ int SIO_ReadSector(int unit, int sector, UBYTE *buffer)
 		ULONG secindex = 0;
 		static int lasttrack = 0;
 		unsigned int currpos, time, delay, rotations, bestdelay;
-		unsigned char beststatus;
+/*		unsigned char beststatus;*/
 		int fromtrack, trackstostep, j;
 
 		info = (vapi_additional_info_t *)additional_info[unit];
@@ -757,7 +758,7 @@ int SIO_ReadSector(int unit, int sector, UBYTE *buffer)
 #endif
 
 		bestdelay = 10 * VAPI_CYCLES_PER_ROT;
-		beststatus = 0;
+/*		beststatus = 0;*/
 		for (j=0;j<secinfo->sec_count;j++) {
 			if (secinfo->sec_rot_pos[j]  < currpos)
 				delay = (VAPI_CYCLES_PER_ROT - currpos) + secinfo->sec_rot_pos[j];
@@ -770,7 +771,7 @@ int SIO_ReadSector(int unit, int sector, UBYTE *buffer)
 #endif
 			if (delay < bestdelay) {
 				bestdelay = delay;
-				beststatus = secinfo->sec_status[j];
+/*				beststatus = secinfo->sec_status[j];*/
 				secindex = j;
 			}
 		}
@@ -845,12 +846,12 @@ int SIO_WriteSector(int unit, int sector, const UBYTE *buffer)
 		secinfo = &info->sectors[sector-1];
 		
 		if (secinfo->sec_count != 1) {
-			// No writes to sectors with duplicates or missing sectors
+			/* No writes to sectors with duplicates or missing sectors */
 			return 'E';
 		}
 		
 		if (secinfo->sec_status[0] != 0xFF) {
-			// No writes to bad sectors
+			/* No writes to bad sectors */
 			return 'E';
 		}
 		
@@ -868,7 +869,7 @@ int SIO_WriteSector(int unit, int sector, const UBYTE *buffer)
 		phantom = &info->phantom[sector-1];
 		
 		if (phantom->phantom_count != 0) {
-			// No writes to sectors with duplicates 
+			/* No writes to sectors with duplicates */
 			return 'E';
 		}
 		
@@ -1234,44 +1235,31 @@ void SIO_Handler(void)
 	}
 	/* cassette i/o */
 	else if (MEMORY_dGetByte(0x300) == 0x60) {
-		int storagelength = 0;
 		UBYTE gaps = MEMORY_dGetByte(0x30b);
 		switch (cmd){
 		case 0x52:	/* read */
-			/* set expected Gap */
-			CASSETTE_AddGap(gaps == 0 ? 2000 : 160);
 			SIO_last_op = SIO_LAST_READ;
 			SIO_last_drive = 0x61;
 			SIO_last_op_time = 0x10;
+			/* set expected Gap */
+			CASSETTE_AddGap(gaps == 0 ? 2000 : 160);
 			/* get record from storage medium */
-			storagelength = CASSETTE_Read();
-			if (storagelength - 1 != length)	/* includes -1 as error */
-				result = 'E';
-			else
+			if (CASSETTE_ReadToMemory(data, length))
 				result = 'C';
-			/* check checksum */
-			if (CASSETTE_buffer[length] != SIO_ChkSum(CASSETTE_buffer, length))
+			else
 				result = 'E';
-			/* if all went ok, copy to Atari */
-			if (result == 'C')
-				MEMORY_CopyToMem(CASSETTE_buffer, data, length);
 			break;
 		case 0x57:	/* write */
 			SIO_last_op = SIO_LAST_WRITE;
 			SIO_last_drive = 0x61;
 			SIO_last_op_time = 0x10;
-			/* put record into buffer */
-			MEMORY_CopyFromMem(data, CASSETTE_buffer, length);
-			/* eval checksum over buffer data */
-			CASSETTE_buffer[length] = SIO_ChkSum(CASSETTE_buffer, length);
 			/* add pregap length */
 			CASSETTE_AddGap(gaps == 0 ? 3000 : 260);
 			/* write full record to storage medium */
-			storagelength = CASSETTE_Write(length + 1);
-			if (storagelength - 1 != length)	/* includes -1 as error */
-				result = 'E';
-			else
+			if (CASSETTE_WriteFromMemory(data, length))
 				result = 'C';
+			else
+				result = 'E';
 			break;
 		default:
 			result = 'N';
@@ -1302,6 +1290,19 @@ void SIO_Handler(void)
 	MEMORY_dPutByte(0x0303, CPU_regY);
 	MEMORY_dPutByte(0x42,0);
 	CPU_SetC;
+
+	/* After each SIO operation a routine called SENDDS ($EC5F in OSB) is
+	   invoked, which, among other functions, silences the sound
+	   generators. With SIO patch we don't call SIOV and in effect SENDDS
+	   is not called either, but this causes a problem with tape saving.
+	   During tape saving sound generators are enabled before calling
+	   SIOV, but are not disabled later (no call to SENDDS). The effect is
+	   that after saving to tape the unwanted SIO sounds are left audible.
+	   To avoid the problem, we silence the sound registers by hand. */
+	POKEY_PutByte(POKEY_OFFSET_AUDC1, 0);
+	POKEY_PutByte(POKEY_OFFSET_AUDC2, 0);
+	POKEY_PutByte(POKEY_OFFSET_AUDC3, 0);
+	POKEY_PutByte(POKEY_OFFSET_AUDC4, 0);
 }
 
 UBYTE SIO_ChkSum(const UBYTE *buffer, int length)
@@ -1479,46 +1480,19 @@ static UBYTE Command_Frame(void)
 /* Enable/disable the Tape Motor */
 void SIO_TapeMotor(int onoff)
 {
-	/* if sio is patched, do not do anything */
-	if (ESC_enable_sio_patch)
-		return;
+	CASSETTE_TapeMotor(onoff);
 	if (onoff) {
 		/* set frame to cassette frame, if not */
 		/* in a transfer with an intelligent peripheral */
-		if (TransferStatus == SIO_NoFrame || (TransferStatus & 0xfe) == SIO_CasRead) {
-			if (CASSETTE_IsSaveFile()) {
-				TransferStatus = SIO_CasWrite;
-				CASSETTE_TapeMotor(onoff);
-				SIO_last_op = SIO_LAST_WRITE;
-			}
-			else {
-				TransferStatus = SIO_CasRead;
-				CASSETTE_TapeMotor(onoff);
-				POKEY_DELAYED_SERIN_IRQ = CASSETTE_GetInputIRQDelay();
-				SIO_last_op = SIO_LAST_READ;
-			};
-			SIO_last_drive = 0x60;
-			SIO_last_op_time = 0x10;
-		}
-		else {
-			CASSETTE_TapeMotor(onoff);
-		}
+		if (TransferStatus == SIO_NoFrame || TransferStatus == SIO_CasReadWrite)
+			TransferStatus = SIO_CasReadWrite;
+		SIO_last_drive = 0x60;
+		SIO_last_op_time = 0x10;
 	}
 	else {
 		/* set frame to none */
-		if (TransferStatus == SIO_CasWrite) {
+		if (TransferStatus == SIO_CasReadWrite)
 			TransferStatus = SIO_NoFrame;
-			CASSETTE_TapeMotor(onoff);
-		}
-		else if (TransferStatus == SIO_CasRead) {
-			TransferStatus = SIO_NoFrame;
-			CASSETTE_TapeMotor(onoff);
-			POKEY_DELAYED_SERIN_IRQ = 0; /* off */
-		}
-		else {
-			CASSETTE_TapeMotor(onoff);
-			POKEY_DELAYED_SERIN_IRQ = 0; /* off */
-		}
 		SIO_last_op_time = 0;
 	}
 }
@@ -1619,7 +1593,7 @@ void SIO_PutByte(int byte)
 			Log_print("Invalid data frame!");
 		}
 		break;
-	case SIO_CasWrite:
+	case SIO_CasReadWrite:
 		CASSETTE_PutByte(byte);
 		break;
 	}
@@ -1674,9 +1648,8 @@ int SIO_GetByte(void)
 			TransferStatus = SIO_NoFrame;
 		}
 		break;
-	case SIO_CasRead:
+	case SIO_CasReadWrite:
 		byte = CASSETTE_GetByte();
-		POKEY_DELAYED_SERIN_IRQ = CASSETTE_GetInputIRQDelay();
 		break;
 	default:
 		break;
