@@ -117,7 +117,8 @@ int PLATFORM_Keyboard(void)
 int PLATFORM_Initialise(int *argc, char *argv[])
 {
 #ifdef SOUND
-	Sound_Initialise(argc, argv);
+	if (!Sound_Initialise(argc, argv))
+		return FALSE;
 #endif
 
 	if (gron(argc, argv))
