@@ -13,14 +13,14 @@
 #define XEP80_FONTS_UNDER_FONT_BIT      0x2
 #define XEP80_FONTS_BLK_FONT_BIT        0x4
 
-#define XEP80_FONTS_UNDER_ROW     10
+#define XEP80_FONTS_UNDER_ROW     9
 
-extern UBYTE XEP80_FONTS_atari_fonts[XEP80_FONTS_NUM_FONT_SETS][XEP80_FONTS_NUM_FONTS][XEP80_FONTS_CHAR_COUNT][XEP80_CHAR_HEIGHT][XEP80_CHAR_WIDTH];
+extern UBYTE XEP80_FONTS_atari_fonts[XEP80_FONTS_NUM_FONT_SETS][XEP80_FONTS_NUM_FONTS][XEP80_FONTS_CHAR_COUNT][XEP80_MAX_CHAR_HEIGHT][XEP80_CHAR_WIDTH];
 extern UBYTE XEP80_FONTS_oncolor;
 extern UBYTE XEP80_FONTS_offcolor;
 
 extern int XEP80_FONTS_inited;
 
-void XEP80_FONTS_InitFonts(void);
+int XEP80_FONTS_InitFonts(char const *charset_filename);
 
 #endif /* XEP80_FONTS_H_ */
